#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <signal.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "../include/ax25.h"
#include "../include/rose.h"
#include "../include/axlib.h"
#include "../include/axconfig.h"
#include "../include/daemon.h"
#include "../include/axconfig.h"
#include "../include/balise.h"


static int logging = FALSE;
/*
char WXDigipath[35]  ={"WX v "};
char APRSDigipath[35]  ={"APRS v "};
*/
static void
terminate (int sig)
{
  if (logging)
    {
      syslog (LOG_INFO, "terminating on SIGTERM\n");
      closelog ();
    }
  exit (0);
}

int
main (int argc, char *argv[])
{
  struct full_sockaddr_ax25 dest;
  struct full_sockaddr_ax25 src;
  int s, n, dlen, len;
  char addr[60];
  char *message = NULL;		/* */
  char *portcall;
  char *Digi = NULL;		/* */
  char *srccall = NULL;
  char *port = NULL;

  while ((n = getopt (argc, argv, "s:p:d:m:")) != -1)
    {
      switch (n)
	{
	case 's':
	  srccall = optarg;
	  break;

	case 'p':
	  port = optarg;
	  break;

	case 'm':
	  message = optarg;
	  break;

	case 'd':
	  Digi = optarg;
	  break;

	case ':':
	  printf
	    ("Syntaxe balise -s<sourcecall> -p<port> -d<Digi> -m<\"message\">\n");
	  return 1;

	}
    }

  signal (SIGTERM, terminate);
  if (argc < 5)
    {
      printf
	("Usage balise -s<sourcecall> -p<port> -d<Digi> -m<\"message\">\n");
      return 1;
    };

  if (ax25_config_load_ports () == 0)
    {
      fprintf (stderr, "beacon: no AX.25 ports defined\n");
      return 1;
    }

  if ((portcall = ax25_config_get_addr (port)) == NULL)
    {
      fprintf (stderr, "beacon: invalid AX.25 port setting\n");
      return 1;
    }
/*	
        strcat (Digipath,Digi);
	strcpy (addr, Digipath);
*/
  strcpy (addr, Digi);

  if ((dlen = ax25_aton (addr, &dest)) == -1)
    {
      fprintf (stderr, "beacon: unable to convert callsign '%s'\n", addr);
      return 1;
    }

  if (srccall != NULL && strcmp (srccall, portcall) != 0)
    sprintf (addr, "%s %s", srccall, portcall);
  else
    strcpy (addr, portcall);

  if ((len = ax25_aton (addr, &src)) == -1)
    {
      fprintf (stderr, "beacon: unable to convert callsign '%s'\n", addr);
      return 1;
    }

  for (;;)
    {
      if ((s = socket (AF_AX25, SOCK_DGRAM, 0)) == -1)
	{
	  return 1;
	}

      if (bind (s, (struct sockaddr *) &src, len) == -1)
	{
	  return 1;
	}

      if (sendto
	  (s, message, strlen (message), 0, (struct sockaddr *) &dest,
	   dlen) == -1)
	{
	  return 1;
	}

      close (s);
      break;
    }

  return 0;
}
